#version 330 core

layout(pixel_center_integer) in vec4 gl_FragCoord; // pixel positions at integer coordinates

uniform vec3 color;
uniform uint colorID;
uniform float farPlane = 200.f;

uniform sampler2D depthMap;
uniform sampler2D depthFrontFace;
uniform mat4 perspective;

uniform vec3 viewLightPosition;

uniform vec2 invImageSize; // 1.0/width, 1.0/height

out vec4 outColor;
out uint outID;

in vec3 vf_normal;
in vec3 vf_viewPosition;
in vec3 vf_color;

void main(void)
{
	// depth of current fragment (0=camera, 1=far plane)
	float depth = -vf_viewPosition.z / farPlane;

	vec2 center = (gl_FragCoord.xy + vec2(0.5,0.5));
	vec2 centerp = center * invImageSize;

	// depth in cutting volume
	float depthMapSample = texture(depthMap, centerp).r;
	
	// if pixel in front of cutting volume -> discard
	if (depth < depthMapSample)
	{
		discard;
	}

	// depth of front face
	float depthFrontFaceSample = texture(depthFrontFace, centerp).r;

	// if back face visible -> project onto cutting volume, otherwise discard
	if (depth < depthFrontFaceSample)
	{
		gl_FragDepth = depthMapSample;
	}
	else
	{
		discard;
	}

	// reconstruct view position of pixel on the cutting volume from depth map
	float viewDepth = -depthMapSample * farPlane;
	vec2 screenPosition = centerp * 2.0 + vec2(-1.0, -1.0);
	vec3 viewPosition = vec3(screenPosition * (-viewDepth) / vec2(perspective[0][0], perspective[1][1]), viewDepth);

	// generate normal and shade
	vec3 n = normalize(cross(dFdx(viewPosition), dFdy(viewPosition)));
	vec3 l = normalize(viewLightPosition - vf_viewPosition);

	float light = clamp(dot(l, n), 0.0, 1.0);

	float lightCell = floor(light*8.0) / 8.0;
	float lightSmaller = lightCell*0.5+0.4;
	
	outColor = vec4(vf_color*color*lightSmaller, 1.0);
	outID = colorID;
}