#version 330 core

layout(pixel_center_integer) in vec4 gl_FragCoord; // pixel positions at integer coordinates

uniform sampler2D texture0; // depth texture
uniform sampler2D depthTexture;
uniform uvec2 k; // jump length
uniform vec2 invImageSize; // 1.0/width, 1.0/height

out vec4 outColor;

void main(void)
{
	vec2 center = (gl_FragCoord.xy + vec2(0.5,0.5));
	vec2 centerp = center * invImageSize;

	vec2 kp = vec2(k) * invImageSize;

	vec4 best = vec4(0.0,0.0,0.0,0.0);
	float newDepthSample = 0.0f;
	float bestDepth = -1.f;

	float offsets[18] = float[18](
		kp.x,kp.y, kp.x,0, kp.x,-kp.y,
		0,kp.y, 0,0, 0,-kp.y,
		-kp.x,kp.y, -kp.x,0, -kp.x,-kp.y
	);

	for (int i=0; i<9; i++)
	{
		vec2 samplePosition = centerp + vec2(offsets[2*i],offsets[2*i+1]);
		float depthSample = texture(depthTexture, samplePosition).r;
		vec4 texture0Sample = texture(texture0, samplePosition);

		vec2 texOther = texture0Sample.xy * 65535.0 + vec2(0.5,0.5);
		vec2 qp = texOther - center;
		float alpha = clamp( dot(normalize(qp), vec2(0.0f,1.0f)), 0.0f, 1.0f);

		float dist = length(qp) * (alpha*texture0Sample.z + (1.0f-alpha)*texture0Sample.w);
		float newDepth = depthSample - dist;
		if (newDepth > bestDepth)
		{
			best = texture0Sample;
			bestDepth = newDepth;
			newDepthSample = depthSample;
		}
	}

	outColor = best;
	gl_FragDepth = newDepthSample;
}