#version 330 core

layout(pixel_center_integer) in vec4 gl_FragCoord;

uniform mat4 modelView;
uniform vec2 imageSize;
uniform float farPlane = 200.f;
uniform float maxTheta;
uniform float edgeCompression;
uniform float directional = 1.f;

in vec3 vf_view;
in vec3 vf_viewPosition;

out vec4 outColor;

void main(void)
{
	float depth = -vf_viewPosition.z / farPlane;

	vec2 screenPosition = (gl_FragCoord.xy / imageSize.xy)*2.f - 1.f;
	float edgeX = clamp(10.f*(1.f-abs(screenPosition.x)), edgeCompression, 1.f);
	float edgeY = clamp(10.f*(1.f-abs(screenPosition.y)), edgeCompression, 1.f);

	float magicMultiplier = depth * 0.63f/(edgeX*edgeY*(imageSize.y/2.f));

	float phip = 3.1415f/2.f - acos( dot( normalize(vf_view), vec3(0.f,1.f,0.f) ) );
	float thetaDirectional = clamp(maxTheta,0.0,phip) + maxTheta*smoothstep(0.f, 3.1415f/18.f, -phip);

	float localmDirectional = (1.0f / tan(thetaDirectional)) * magicMultiplier;
	float localm = (1.0f / tan(maxTheta)) * magicMultiplier;

	localmDirectional = directional * localmDirectional + (1.0f-directional)*localm;

	outColor = vec4(gl_FragCoord.x/65535.0, gl_FragCoord.y/65535.0, localmDirectional, localm);
	gl_FragDepth = depth;
}